import 'package:flutter/material.dart';
import 'package:frebu_example/in_memory_repository.dart';

import 'note.dart';
import 'repository.dart';

void main() => runApp(MyApp());

class MyApp extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return MaterialApp(
      title: 'Frebu Demo',
      theme: ThemeData(
        primarySwatch: Colors.blue,
      ),
      home: MyHomePage(),
    );
  }
}

class MyHomePage extends StatefulWidget {
  MyHomePage({Key key}) : super(key: key);

  @override
  _MyHomePageState createState() => _MyHomePageState();
}

class _MyHomePageState extends State<MyHomePage> {
  TextEditingController controller;
  NotesRepository repository;

  @override
  void initState() {
    super.initState();
    controller = TextEditingController();
    repository = InMemoryNotesRepository();
    repository.execute(FetchNotes());
  }

  @override
  void dispose() {
    controller.dispose();
    repository.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(title: Text('Notes')),
      body: Center(
        child: Column(
          mainAxisAlignment: MainAxisAlignment.center,
          children: <Widget>[
            Expanded(
                child: StreamBuilder<List<Note>>(
                    stream: repository.notesStream,
                    initialData: [],
                    builder: (context, snapshot) {
                      final notes = snapshot.data;
                      return ListView.builder(
                        itemCount: notes.length,
                        itemBuilder: (context, i) => ListTile(
                          title: Text(notes[i].message),
                        ),
                      );
                    })),
            Divider(),
            TextField(
              controller: controller,
              textInputAction: TextInputAction.next,
              onSubmitted: (message) {
                repository.execute(AddNote(message));
                controller.clear();
              },
            )
          ],
        ),
      ),
    );
  }
}
