import 'package:flutter/foundation.dart';
import 'package:frebu/frebu.dart';
import 'package:uuid/uuid.dart';
import 'package:rxdart/rxdart.dart';

import 'note.dart';

abstract class NotesRepository extends Repository {
  NotesRepository() : emitter = NotesEmitter();

  @protected
  NotesEmitter emitter;

  Observable<List<Note>> get notesStream => emitter.stream;

  @override
  void dispose() {
    emitter.dispose();
    super.dispose();
  }
}

class AddNote extends Action<bool> {
  AddNote(this.message) : id = Uuid().v4();
  final String message;
  final String id;
}

class FetchNotes extends Action<List<Note>> {
  const FetchNotes();
}

class NotesEmitter extends PublishSubjectEmitter<List<Note>> {
  List<Note> latestValue;

  Future<void> onNoteAdd(AddNote action, bool result) async {
    latestValue.add(Note(action.id, action.message));
    safeAdd(latestValue);
  }

  Future<void> onFetchNotes(FetchNotes action, List<Note> result) async {
    latestValue = result;
    safeAdd(latestValue);
  }
}
